import React, { Component } from 'react';
import {Typography, Switch} from '@material-ui/core';
import {
    BlockFormControlLabel,
    SectionContent,
    webSocketController,
    WebSocketControllerProps,
    WebSocketFormLoader,
    WebSocketFormProps
} from '../components';

import {InformationState} from "./types";
import {WEB_SOCKET_ROOT} from "../api";
import {ValidatorForm} from "react-material-ui-form-validator";

export const INFORMATION_WEBSOCKET_URL = WEB_SOCKET_ROOT + "information";
type InformationStateWebSocketControllerProps = WebSocketControllerProps<InformationState>;

class DemoInformation extends Component<InformationStateWebSocketControllerProps> {

  render() {
    return (
      <SectionContent title='WordClock Information' titleGutter>
        <Typography variant="body1" paragraph>
          Welcome to the WordClock Overview page. Here you can turn on and off your WordClock and set Basic Settings
        </Typography>
        <WebSocketFormLoader
          {...this.props}
          render={props => (
              <LightStateWebSocketControllerForm {...props} />
          )}
      />
      </SectionContent>
    )
  }
}

export default webSocketController(INFORMATION_WEBSOCKET_URL, 100, DemoInformation);

type InformationWebSocketControllerFormProps = WebSocketFormProps<InformationState>;

function LightStateWebSocketControllerForm(props: InformationWebSocketControllerFormProps) {
    const { data, saveData, setData } = props;

    const changeWordClockOn = (event: React.ChangeEvent<HTMLInputElement>) => {
        setData({wordclockOn: event.target.checked,twentyAfterSyntax: data.twentyAfterSyntax, uhrAlwaysOn: data.uhrAlwaysOn }, saveData);
    }

    const changeTwentyAfterSyntax = (event: React.ChangeEvent<HTMLInputElement>) => {
        setData({wordclockOn: data.wordclockOn, twentyAfterSyntax: event.target.checked, uhrAlwaysOn: data.uhrAlwaysOn }, saveData);
    }

    const changeUhrAlwaysOn = (event: React.ChangeEvent<HTMLInputElement>) => {
        setData({wordclockOn: data.wordclockOn,twentyAfterSyntax: data.twentyAfterSyntax, uhrAlwaysOn: event.target.checked }, saveData);
    }

    return (
        <ValidatorForm onSubmit={saveData}>
            <BlockFormControlLabel
                control={
                    <Switch
                        checked={data.wordclockOn}
                        onChange={changeWordClockOn}
                        color="primary"
                    />
                }
                label="WordClock turned on?"
            />
            <BlockFormControlLabel
                control={
                    <Switch
                        checked={data.twentyAfterSyntax}
                        onChange={changeTwentyAfterSyntax}
                        color="primary"
                    />
                }
                label="Use Twenty-After Syntax?"
            />
            <BlockFormControlLabel
                control={
                    <Switch
                        checked={data.uhrAlwaysOn}
                        onChange={changeUhrAlwaysOn}
                        color="primary"
                    />
                }
                label="Always Display UHR label"
            />
        </ValidatorForm>
    );
}
