//
// Created by lukas on 06.03.21.
//

#include "Types.h"
#include "Clock.h"

Clock::Clock() : strip(NUMPIXELS, STRIPPIN, NEO_GRB + NEO_KHZ800), refreshTicker() {
  LoadAnimator an = LoadAnimator();
  animator =  &an;
}

void Clock::init() {
  strip.begin();
  strip.setBrightness(DAYBRIGHTNESS);

  // initialize clock ticker
  turnOn();
}

void Clock::turnOff() {
  strip.clear();
  strip.show();

  refreshTicker.detach();
}

void Clock::turnOn() {
  if (!refreshTicker.active())
    refreshTicker.attach(10, [this]() { this->refreshTime(); });
  this->refreshTime();
}

void Clock::paintAlwaysOnLeds() {
  addque.push_back(Word(types::es, 0xFFFF00));
  addque.push_back(Word(types::ist, 0xFF00FF));
}

void Clock::printWord(const std::vector<uint8_t>& word, uint32_t color, bool highlightfirst) {
  uint r = (color >> 16) & 0xFF;
  uint g = (color >> 8) & 0xFF;
  uint b = (color)&0xFF;

  for (const uint8_t& i : word) {
    strip.setPixelColor(
        i,
        !highlightfirst || (&i == &word.back())
            ? color
            : (((r > 200 ? r - 200 : r) << 16) | ((g > 200 ? g - 200 : g) << 8) | (b > 200 ? b - 200 : b)));
  }
}

void Clock::refreshTime() {
  // grab the current instant in unix seconds
  time_t now = time(nullptr);

  // if not time is set (only seconds since start) start animation
  if (now <= 604800) {
    animator->startLoadAnimation(&strip);
    return;
  }

  if (animator->animationActive()) {
    animator->stopLoadAnimation();
    strip.clear();

    // only add on first time iteration
    paintAlwaysOnLeds();
  }

  tm* loctime = localtime(&now);

  const uint8_t hour = loctime->tm_hour;
  const uint8_t minute = loctime->tm_min;

  // enable night brighntess at 21' and disable it at 6'
  bool daybrightness = !(hour >= 22 || hour <= 6);

  if (oldDayBrightness != daybrightness) {
    strip.setBrightness(daybrightness ? DAYBRIGHTNESS : NIGHTBRIGHTNESS);
    paintAlwaysOnLeds();

    oldDayBrightness = daybrightness;
  }

  setTime(hour, minute, this->twentyAfterSyntax, this->clockAlwaysOn);

  strip.show();
  Serial.printf("Time now: %uh %uM\n", hour, minute);
}

void Clock::setTime(uint8_t hour, uint8_t minute, bool twas, bool alwaysClockWord) {
  const uint8_t minuteselector = minute / 5;

  // if minuteselector >= 4 +1 to hour
  if (twas ? minuteselector >= 5 : minuteselector >= 4)
    hour++;

  // convert to 12h format
  if (hour >= 12)
    hour = hour - 12;

  std::vector<uint8_t> hourWord;
  hourWord = hour == 1                 ? (minuteselector == 0 ? types::ein : types::eins)  // eins only on a full hour
             : hour == 2               ? types::zwei
             : hour == 3               ? types::drei2
             : hour == 4               ? types::vier
             : hour == 5               ? types::fuenf2
             : hour == 6               ? types::sechs
             : hour == 7               ? types::sieben
             : hour == 8               ? types::acht
             : hour == 9               ? types::neun
             : hour == 10              ? types::zehn2
             : hour == 11              ? types::elf
             : hour == 0 || hour == 12 ? types::zwoelf
                                       : hourWord;

  if (oldhourWord != hourWord) {
    addque.push_back(Word(hourWord, 0x00FFFF));
    delque.push_back(Word(oldhourWord, 0x00FFFF));

    oldhourWord = hourWord;
  }

  // print the minute words and the corresponding after/before half words
  printMinutes(minuteselector, twas);

  // uhr
  bool uhractive = minuteselector == 0 || alwaysClockWord;

  if (oldUhrActive != uhractive) {
    const Word wrd = Word(types::uhr, 0xFF0000);
    (uhractive ? addque : delque).push_back(wrd);
    oldUhrActive = uhractive;
  }

  performWordTransition();
}

void Clock::printMinutes(uint8_t minuteSelector, bool twas) {
  // fuenf / zehn / viertl word
  std::vector<uint8_t> minuteWord, vornachWord;
  bool halfActive = false;

  if (minuteSelector == 1 || minuteSelector == 5 || minuteSelector == 7 || minuteSelector == 11) {
    minuteWord = types::fuenf;
  } else if (minuteSelector == 2 || minuteSelector == 4 || minuteSelector == 8 || minuteSelector == 10) {
    // check if we use the twenty after syntax
    if ((minuteSelector == 4 || minuteSelector == 8) && twas)
      minuteWord = types::zwanzig;
    else
      minuteWord = types::zehn;
  } else if (minuteSelector == 3)
    minuteWord = types::viertel;
  else if (minuteSelector == 9)
    minuteWord = types::dreiviertel;

  if (oldminuteWord != minuteWord) {
    addque.push_back(Word(minuteWord, 0x00FF00));
    delque.push_back(Word(oldminuteWord, 0x00FF00));
    oldminuteWord = minuteWord;
  }

  // vor / nach
  if (minuteSelector == 1 || minuteSelector == 2 || minuteSelector == 3 ||
      ((minuteSelector == 8) && !twas) ||  // check if twentyafter syntax is enabled - if not display the the after
      minuteSelector == 7 ||
      ((minuteSelector == 4) && twas))  // check if twentyafter syntax is enabled - if yes display the the after
    vornachWord = types::nach;
  else if (minuteSelector == 5 ||
           ((minuteSelector == 4) && !twas)  // check if twentyafter enabled if not -- display vor
           || minuteSelector == 10 || minuteSelector == 11 ||
           ((minuteSelector == 8) && twas))  // check if twentyafter enabled if yess -- display vor
    vornachWord = types::vor;

  if (oldvornachWord != vornachWord) {
    addque.push_back(Word(vornachWord, 0xFFFFFF));
    delque.push_back(Word(oldvornachWord, 0xFFFFFF));
    oldvornachWord = vornachWord;
  }

  // halb
  if (minuteSelector >= 4 && minuteSelector <= 8) {
    // only 3 times if twentyafter syntax is on
    if (!twas || (minuteSelector >= 5 && minuteSelector <= 7))
      halfActive = true;
  }

  if (oldhalfActive != halfActive) {
    (halfActive ? addque : delque).push_back(Word(types::halb, 0xFF0000));
    oldhalfActive = halfActive;
  }
}

void Clock::useTwentyAfterSyntax(bool twas) {
  this->twentyAfterSyntax = twas;
}

void Clock::useAlwaysOnUhrSyntax(bool alwaysOnUhr) {
  this->clockAlwaysOn = alwaysOnUhr;
}

void Clock::update() {
  if (refreshTicker.active())
    this->refreshTime();
}

int it = 0;
void Clock::performWordTransition() {
  if (addque.empty() && delque.empty())
    return;

  int maxdels = 0;
  for (const Word& el : delque) {
    if (el.wrd.size() > maxdels) {
      maxdels = el.wrd.size();
    }
  }

  int maxadds = 0;
  for (const Word& el : addque) {
    if (el.wrd.size() > maxadds) {
      maxadds = el.wrd.size();
    }
  }

  //  const uint iterationcount = maxdels + maxadds;

  if (!transitionTicker.active()) {
    transitionTicker.attach_ms(150, [this, maxdels, maxadds]() {
      if (it <= maxdels) {
        for (Word word : delque) {
          if (word.wrd.size() < it)
            continue;

          this->printWord(word.wrd, 0x0, false);
          this->printWord({word.wrd.begin(), word.wrd.end() - it}, word.color, false);
        }

        this->strip.show();
      } else if (it - maxdels <= maxadds) {
        // we are modding the endword
        int idx = it - maxdels;
        for (Word word : addque) {
          if (word.wrd.size() < idx)
            continue;

          this->printWord(word.wrd, 0x0, false);
          this->printWord({word.wrd.begin(), word.wrd.begin() + idx}, word.color, idx < word.wrd.size());
        }
        this->strip.show();
      } else {
        addque.clear();
        delque.clear();

        transitionTicker.detach();
        it = 0;
      }

      it++;
    });
  }
}
